#!/bin/sh
#
# Component: log
#
# Description:
#    Collect log entries between specific dates.
#
# Usage:
#
#    viewlog [-s start] [-e end] [-a action1[,action2...]] [-l level1[,level2...]]
#
#       start and end is either - (for today) or date day for older files
#       start must be <= end
#       actions can be set for filtering.
#       levels can be set for filtering.
#
#       Date format is "YYYY-MM-DD HH:MM:SS"
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

usage()
{
	cat <<-END_OF_USAGE >&2
		usage: viewlog.sh [-s start] [-e end] [-a action1[,action2...]] [-l level]
		   -s start                 : start date, '-' means today 00:00:00 (default : '-')
		   -e end                   : end date,   '-' means today 23:59:59 (default : '-')
		   -a action1[,action2...]  : output only these actions (default: show everything)
		   -l level1[,level2...]    : output only this levels   (default: show everything) 
		END_OF_USAGE
	exit
} # usage

#. /usr/develop/etc/setenv.sh
. $HOME/etc/setenv.sh

startDT=""
endDT=""
actions=""
levels=""

while getopts 's:e:a:l:h' OPT; do
	case $OPT in
	s) startDT="$OPTARG" ;;
	e) endDT="$OPTARG" ;;
	a) actions="$OPTARG" ;;
	l) levels="$OPTARG" ;;
	h) usage ;;
	*) usage ;;
	esac
done

[ -z "$startDT" -o "$startDT" = "-" ] && startDT=`date +"%Y-%m-%d 00:00:00"`
[ -z "$endDT" -o "$endDT" = "-" ] && endDT=`date +"%Y-%m-%d 23:59:59"`

[ `expr length "$startDT"` = 10 ] && startDT="$startDT 00:00:00"
[ `expr length "$endDT"` = 10 ] && endDT="$endDT 23:59:59"

endYear=`date -d "$endDT" +"%Y"`

fileEndD=`date -d "$endDT + 1 day" +"%Y-%m-%d"`
fileStartD=`date -d "$startDT" +"%Y-%m-%d"`

# new code to pre-filter with grep (actions contains actions like act1,act2...)
if [ -z "$actions" ] ; then
	ACTGREP=cat;
else
	ACTGREP=`echo $actions | awk -v RS="," '
			{ s=s " -e \"" $1 "\""; }
			END { print "grep " s; }'`
fi

HOST=`/bin/hostname -s`
ls $LOGS_DATED/forever | \
	awk -vFS='.' -vStart=$fileStartD -vEnd=$fileEndD '($4 == "oper_info" && $2 >= Start && $2 <= End) { print $2 " " $0}' | \
	(while read fdate fname; do
		# year of the log file date/time
		fyear=`date -d "$fdate" +"%Y"`
		# year of the previous day
		pfyear=`date -d "$fdate -1 day" +"%Y"`

		if [ "$fyear" != "$pfyear" ] ; then
			# New Year - prepend new year before Jan entries and prepend old year before other entries
			zcat $LOGS_DATED/forever/$fname | eval "$ACTGREP" | sed -e "s;^Jan ;$fyear Jan ;" | sed "s;^\([^0-9]* \);$pfyear \1;"
			continue;
		fi

		zcat $LOGS_DATED/forever/$fname | eval "$ACTGREP" | sed 's/^/'$fyear' /'
	done

	[ `date -d "$endDT" +"%s"` -ge `date -d "00:00 today" +"%s"` ] && eval "$ACTGREP $LOG/oper_info.log" | sed 's/^/'$endYear' /'

	) | \
	awk '
		BEGIN {
			lastmsg="-";
		} 
		( $6 == "last" && $7 == "message" && $8 == "repeated" && $10 == "times" ) { 
			if (lastmsg != "-") {
				for (i = 0 ; i < $9 ; i++)
					print $1 " " $2 " " $3 " " $4 lastmsg;
			}
			next;
		}
		{
			print $0;
			lastmsg = "";
			for (i = 5; i <= NF ; i++)
				lastmsg=lastmsg " " $i;
		}' | \
	awk -vActions=$actions -vLevels=$levels -vStart="$startDT" -vEnd="$endDT" ' 
		function cmptime(time1, time2) {
			split(time1,t1,":");
			split(time2,t2,":");
			if (t1[1] - t2[1] != 0)
				return t1[1] - t2[1];
			if (t1[2] - t2[2] != 0)
				return t1[2] - t2[2];
			return t1[3] - t2[3];
		}
		function cmpdate(year,month,day,_time,dt) { 
			if (year - dt[1] != 0)
				return year - dt[1];
			if (m[month] - dt[2] != 0)
				return m[month] - dt[2];
			if (day - dt[3] != 0)
				return day - dt[3];
			return cmptime(_time,dt[4]);
		}
		BEGIN {
			actPattern = "," Actions ",";
			allActions=0;
			if (Actions == "")
				allActions=1;
			levPattern = "," Levels ",";
			allLevels=0;
			if (Levels=="")
				allLevels=1;
			split(Start, sdt, /[- ]/);
			split(End, edt, /[- ]/);
			m["Jan"] = 1; m["Feb"] = 2; m["Mar"] = 3; m["Apr"] = 4;  m["May"] = 5;  m["Jun"] = 6;
			m["Jul"] = 7; m["Aug"] = 8; m["Sep"] = 9; m["Oct"] = 10; m["Nov"] = 11; m["Dec"] = 12;
		}
		( $8 == "OBJECT" || $8 == "STUDY" || $8 == "SERIES" || $8 == "ADMIN" || $8 == "OTHER" || $8 == "TIME") {
			if (!allActions && index(actPattern,"," $9 ",") == 0)
				next;
			if (!allLevels && index(levPattern,"," $8 ",") == 0)
				next;
			if (cmpdate($1,$2,$3,$4,sdt) >= 0 && cmpdate($1,$2,$3,$4,edt) <= 0)
				print;
			next;
		}
		{
			if (!allActions && index(actPattern,"," $8 ",") == 0 )
				next;
			if (!allLevels && index(levPattern,",OTHER,") == 0)
				next;
			if (cmpdate($1,$2,$3,$4,sdt) >= 0 && cmpdate($1,$2,$3,$4,edt) <= 0) {
				for (i = 1 ; i<NF; i++) {
					printf "%s ", $i;
					if (i == 7)
						printf "? ";
				}
				print $NF;
			}
		}'

